<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Comment controller class file.
 */
class CommentController extends BaseController
{

    /**
     * Returns a list of external action classes.
     *
     * @return array
     */
    public function actions()
    {
        return (isset($_POST['ajax']) && $_POST['ajax'] === 'comment-form') ? array() : array(
            'captcha' => array(
                'class' => 'CCaptchaAction',
                'backColor' => 0xFFFFFF,
                'fixedVerifyCode' => YII_DEBUG ? 'polomo' : null
            ),
        );
    }

    /**
     * Get page url on which comments widget was placed
     *
     * @return string
     */
    protected function getWidgetPageUrl()
    {
        if (isset($_GET['return_url'])) {
            return Yii::app()->createAbsoluteUrl($_GET['return_url']);
        } elseif (isset($_SERVER['HTTP_REFERER'])) {
            return $_SERVER['HTTP_REFERER'];
        } else {
            return '';
        }

    }

    /**
     * Intermediate action. Used for saving $return_url in session to return back after log in.
     *
     * @param string $return_url Return url
     *
     * @return void
     */
    public function actionLogin($return_url)
    {
        Yii::app()->user->setReturnUrl($return_url);
        $this->redirect($this->module->loginUrl);
    }

    /**
     * Create Comment action.
     * Reads widget options from session using options hash key. Using options to make validation.
     *
     * @return void
     */
    public function actionCreate()
    {
        if (isset($_POST['key'])) {
            $options = Yii::app()->cache->get($_POST['key']);
        }

        if (!empty($options)) {
            if (!$options['allowAnonymous'] && Yii::app()->user->isGuest) {
                Yii::app()->user->setFlash('commentCreateError' . $options['contentId'], CommentsModule::t('Anonymous comments not allowed'));
            } elseif (isset($_POST['Comment'])) {
                $comment = new Comment(Yii::app()->user->isGuest ? 'createAnonymous' : 'create');
                $comment->attributes = $_POST['Comment'];
                $comment->content_id = $options['contentId'];
                $comment->notify_moderator = $options['notifyModeratorOnNew'];
                $comment->moderator_email = $options['moderatorEmail'];
                $comment->status = $options['dontNeedApprove'] ? Comment::APPROVED : Comment::NOT_APPROVED;
                $comment->language = Yii::app()->language;
                if (!Yii::app()->user->isGuest) {
                    $comment->user_id = Yii::app()->user->id;
                    $comment->email = Yii::app()->user->email;
                }
                if ($comment->save()) {
                    if ($options['dontNeedApprove']) {
                        Yii::app()->user->setFlash('commentCreateSuccess' . $options['contentId'], CommentsModule::t('Your comment added successfully and will be displayed'));
                    } else {
                        Yii::app()->user->setFlash('commentCreateSuccess' . $options['contentId'], CommentsModule::t('Your comment added successfully and will be visible after approval of a moderator'));
                    }

                    Yii::app()->user->setState('Comment' . $options['contentId'], null);
                    $this->onCommentCreated($comment);
                }
                if ($comment->hasErrors() && !isset($_POST['ajax'])) {
                    Yii::app()->user->setState('Comment' . $options['contentId'], array('attributes' => $comment->attributes, 'errors' => $comment->getErrors()));
                }
            }
        } else {
            Yii::app()->user->setFlash('commentCreateError', CommentsModule::t('Form error'));
        }
        $this->redirect($_GET['return_url']);
    }

    /**
     * Delete comment
     *
     * @param int    $id         comment id
     * @param string $key        options hash key
     * @param string $return_url return url
     *
     * @throws CHttpException
     *
     * @return void
     */
    public function actionDelete($id, $key, $return_url)
    {
        $options = Yii::app()->cache->get($key);

        if (!empty($options) && !Yii::app()->user->isGuest) {
            $comment = Comment::model()->findByPk($id);
            if ($comment && CommentsModule::canDelete($comment)) {
                if ($comment->delete()) {
                    $this->onCommentDeleted($comment);
                    Yii::app()->user->setFlash('commentDeleteSuccess' . $options['contentId'], CommentsModule::t("Comment deleted successfully"));
                }
            } else {
                Yii::app()->user->setFlash('commentDeleteError' . $options['contentId'], CommentsModule::t("You have no access or comment deleted"));
            }
        } else {
            Yii::app()->user->setFlash('commentDeleteError' . $options['contentId'], CommentsModule::t("You have no access"));
        }
        $this->redirect($return_url);
    }

    /**
     * Delete comment be not logged in. Mainly using in admin mails.
     *
     * @param int    $id  comment id
     * @param string $key access hash key
     *
     * @return void
     * @throws CHttpException
     */
    public function actionRemoteDelete($id, $key)
    {
        $comment = Comment::model()->resetScope()->findByPk($id);
        if ($comment && $comment->hash() == $key) {
            if ($comment->status == Comment::DELETED) {
                Yii::app()->user->setFlash('remoteActionState', CommentsModule::t('Comment already deleted'));
            } else {
                $comment->delete();
                $this->onCommentDeleted($comment);
                Yii::app()->user->setFlash('remoteActionState', CommentsModule::t('Comment deleted'));
            }
        }else{
            Yii::app()->user->setFlash('remoteActionState', CommentsModule::t('You have no access'));
            $comment = false;
        }
        $this->render('remoteAction', array('comment' => $comment));
    }


    /**
     * Approve comment
     *
     * @param int    $id         comment id
     * @param string $key        options hash key
     * @param int    $status     flag Comment::NOT_APPROVED|Comment::APPROVED. Specify approve comment or not
     * @param string $return_url return url
     *
     * @return void
     */
    public function actionApprove($id, $key, $status, $return_url)
    {
        $options = Yii::app()->cache->get($key);
        if (!empty($options) && !Yii::app()->user->isGuest && in_array($status, array(Comment::APPROVED, Comment::NOT_APPROVED))) {
            $comment = Comment::model()->findByPk($id);
            if ($comment && CommentsModule::canApprove($comment)) {
                $comment->status = $status;
                $comment->update('status');
                Yii::app()->user->setFlash(
                    'commentApproveSuccess' . $options['contentId'],
                    CommentsModule::t('Comment ' . ($status == Comment::NOT_APPROVED ? 'not' : '') . ' approved')
                );
            } else {
                Yii::app()->user->setFlash('commentApproveError' . $options['contentId'], CommentsModule::t("You have no access"));
            }
        } else {
            Yii::app()->user->setFlash('commentApproveError' . $options['contentId'], CommentsModule::t("You have no access or comment deleted"));
        }
        $this->redirect($return_url);
    }

    /**
     * Approve comment not logged in. Mainly using in admin mails.
     *
     * @param int    $id     comment id
     * @param string $key    access hash key
     * @param int    $status flag Comment::NOT_APPROVED|Comment::APPROVED. Specify approve comment or not
     *
     * @return void
     * @throws CHttpException
     */
    public function actionRemoteApprove($id, $key, $status)
    {
        $comment = Comment::model()->resetScope()->findByPk($id);
        if ($comment && $comment->hash() == $key) {
            if ($comment->status == Comment::DELETED) {
                Yii::app()->user->setFlash('remoteActionState', CommentsModule::t('Comment already deleted'));
            } elseif (in_array($status, array(Comment::APPROVED, Comment::NOT_APPROVED))) {
                $comment->status = $status;
                $comment->update('status');
                Yii::app()->user->setFlash('remoteActionState', CommentsModule::t('Comment ' . ($comment->status == Comment::NOT_APPROVED ? 'not' : '') . ' approved'));
            }
        }else{
            $comment = false;
            Yii::app()->user->setFlash('remoteActionState', CommentsModule::t('Comment is not available anymore'));
        }
        $this->render('remoteAction', array('comment' => $comment));
    }


    /**
     * This method invoked when new comment created
     *
     * @param Comment $comment comment
     *
     * @return void
     */
    public function onCommentCreated(Comment $comment)
    {
        $pageUrl = $this->getWidgetPageUrl();
        if ($comment->notify_moderator && !empty($comment->moderator_email)) {
            $subject = CommentsModule::t('Comment created (Moderator) {subject}', array('{subject}' => $comment->subject));
            $notification = new Notification(
                $subject,
                $this->renderMail('/mail_templates/comment_created_admin', $subject, array('comment' => $comment, 'pageUrl' => $pageUrl), true),
                array(new EmailRecipient($comment->moderator_email, $this->module->notificationsEmail))
            );
            Yii::app()->notificationManager->notifyAbout($notification);
        }

        $subscribers = Yii::app()->db->createCommand()
            ->select('email')
            ->from('{{comments}}')
            ->where('content_id = :content_id AND notify_user = :notify_user AND status != :status', array(':content_id' => $comment->content_id, ':notify_user' => true, ':status' => Comment::DELETED))
            ->group('email')->queryAll();
        $recipients = array();
        $fromEmail = $this->module->notificationsEmail;
        foreach ($subscribers as $subscriber) {
            if (trim($subscriber["email"]) != ""){
                $recipients[] = new EmailRecipient($subscriber['email'], $fromEmail);
            }
        }
        if (sizeof($recipients) > 0) {
            $subject = CommentsModule::t('Comment created: {subject}', array('{subject}' => $comment->subject));
            $notification = new Notification(
                $subject,
                $this->renderMail('/mail_templates/comment_created_user', $subject, array('comment' => $comment, 'pageUrl' => $pageUrl), true),
                $recipients
            );

            Yii::app()->notificationManager->notifyAbout($notification);
        }

    }

    /**
     * This method invoked when comment deleted
     *
     * @param Comment $comment comment
     *
     * @return void
     */
    public function onCommentDeleted(Comment $comment)
    {
        if ($comment->notify_user) {
            $pageUrl = $this->getWidgetPageUrl();
            $subject = CommentsModule::t('Comment was deleted');
            $notification = new Notification(
                $subject,
                $this->renderMail('/mail_templates/comment_deleted', $subject, array('comment' => $comment, 'pageUrl' => $pageUrl), true),
                array(new EmailRecipient($comment->email, $this->module->notificationsEmail))
            );
            Yii::app()->notificationManager->notifyAbout($notification);
        }
    }
}
