<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Class for loading content from Dynamic Content Renderer
 */
class DynamicContentSource extends ContentSource
{
    /**
     * @var string folder into which content will be cached
     */
    public $cacheFolder;

    private $_modTimeCachePrefix = '__modTime__';

    /**
     * Init Gentics Connector API
     */
    public function init()
    {

    }

    /**
     * Return array of attributes which will be requested from API
     *
     * @return array array of attributes
     */
    public function getRequestedAttributes()
    {
        $mainAttributes = array(
            'contentid',
            'name',
            'filename',
            'updatetimestamp',
            'languagecode',
            'mimetype',
            'content'
        );
        if ($this->usePersonalisation) {
            return array_merge($mainAttributes, $this->personalisationFields);
        }
        return $mainAttributes;
    }

    /**
     * Get content absolute path.
     * Checking if we have newest version on content,
     * if yes return, else download and store
     *
     * @param string $path content path
     *
     * @return \Content|mixed object contained file path, false if error
     */
    public function getContent($path)
    {
        if ($this->isCachedObjectStillValid($path)) {
            return $this->loadObjectContentFromCache($path);
        } else {
            return $this->fetchAndCacheObjectByPath($path);
        }
    }

    /**
     * Get content absolute path.
     * Checking if we have newest version on content,
     * if yes return, else download and store
     *
     * @param string $path content path
     *
     * @return \Content|mixed object contained file path, false if error
     */
    public function getContentById($contentId, $path = null)
    {
        $objectPath = $path;
        if (empty($objectPath)) {
            // Since we only have the objects content ID, we have to
            // fetch its path from the API first.
            $repositoryObject = $this->getRepositoryApi()->fetchObject(
                        $contentId, array_merge($this->getRequestedAttributes(), array("url")));

            if ($repositoryObject === null) {
                return null;
            }

            $objectPath = $repositoryObject['attributes']['url'];
        }

        if ($this->isCachedObjectStillValid($objectPath)) {
            return $this->loadObjectContentFromCache($objectPath);
        } else {
            return $this->fetchAndCacheObjectById($contentId);
        }
    }

    public function getStartPage($folderPath, $locale)
    {
        $localizedAttributeName = Yii::app()->getModule('contentSource')->localizedAttributeName. $locale;

        $folderObject = $this->getRepositoryApi()->getContentAttributes($folderPath, array($localizedAttributeName));
        if($folderObject){
            return $folderObject[$localizedAttributeName];
        }
        return false;
    }

    /**
     * This method takes content from API and save locally
     *
     * @param string $path content path
     *
     * @return string
     */
    protected function fetchAndCacheObjectById($contentId)
    {
        // if refresh is set
        $repositoryObject = $this->getRepositoryApi()->fetchObject(
                    $contentId, array_merge($this->getRequestedAttributes(), array("url")));

        if ($repositoryObject === null) {
            return null;
        }

        return $this->cacheObject($repositoryObject,
                $repositoryObject['attributes']['url']);
    }

    /**
     * This method takes content from API and save locally
     *
     * @param string $path content path
     *
     * @return string
     */
    protected function fetchAndCacheObjectByPath($path)
    {
        // if refresh is set
        $repositoryObject = $this->getRepositoryApi()->getCmsObjectCanBeExternal(
                    $path, $this->getRequestedAttributes());

        if ($repositoryObject === null) {
            return null;
        }

        return $this->cacheObject($repositoryObject, $path);
    }

    /**
     * This method takes content from API and save locally
     *
     * @param string $path content path
     *
     * @return bool|string
     */
    protected function cacheObject($repositoryObject, $path)
    {
        $newRepositoryObject = $repositoryObject;
        // if refresh is set
        if (Yii::app()->request->getParam('refresh') == true) {
            // clear cache of object in CCR
            $this->getRepositoryApi()->clearCache($newRepositoryObject['attributes']['contentid']);

            $newRepositoryObject = $this->getRepositoryApi()->getCmsObjectCanBeExternal(
                $path, $this->getRequestedAttributes());
        }

        // create new cache file
        $this->_createFile($path, $newRepositoryObject['attributes']['content'],
                $newRepositoryObject['attributes']['updatetimestamp']);
        // create new content object
        $content = new Content($this, $path);
        unset($newRepositoryObject['attributes']['content']);
        $content->setAttributes($newRepositoryObject['attributes']);
        return $content;
    }

    /**
     * Check if the cached file is still valid (not expired)
     *
     * @param string $path content path
     *
     * @return bool
     */
    protected function isCachedObjectStillValid($path)
    {
        // path alone seems to be not enough, host must be added to the cache key TODO
        // get path of file in fs cache
        $contentFile = $this->contentFile($path);

        // create change key, add cachefolder in case you have different domains and equal paths
        $pathCacheKey = $this->cacheFolder.$path;

        // if file is not in fs cache, then get from api
        if (!file_exists($contentFile) || Yii::app()->request->getParam('refresh', false)) {
            return false;
        }

        // if no cache of modification timestamp of file is set OR an editorsession is active and the session is not 0, then request new update timestamp of file in CCR
        // we don't cache in case a CMS editor is in the portal  because of instant publishing
        if ( ( Yii::app()->cache->get("lastcheck_".$this->_modTimeCachePrefix . $pathCacheKey) === false ) || 
                    ( Yii::app()->session->get('feSid',false) && Yii::app()->session->get('feSid', false) ) ) {

                // set cache for last modification timestamp of file only for a short time (to prevent reloading)
                Yii::app()->cache->set("lastcheck_".$this->_modTimeCachePrefix . $pathCacheKey , time(), Yii::app()->getModule('contentSource')->cacheTime);
            
                // get modification timestamp from CCR for this file
                if ($lastModificationTime = $this->getRepositoryApi()->getLastModificationTime($path)) {
                        Yii::trace( "$lastModificationTime " . Yii::app()->cache->get($this->_modTimeCachePrefix . $pathCacheKey) . " " . filemtime($this->contentFile($path)), "myoutput");
                        //return true if timestamp from cache is the same as timestamp from CCR for this path OR if modification timestamp of file in fs cache is the same as timestamp of CCR         //return Yii::app()->cache->get("lastcheck_".$this->_modTimeCachePrefix . $path) == $lastModificationTime || filemtime($this->contentFile($path)) == $lastModificationTime;
                        if ( Yii::app()->cache->get($this->_modTimeCachePrefix . $pathCacheKey) == $lastModificationTime || filemtime($this->contentFile($path)) == $lastModificationTime ) {
                                return true;
                        } else {
                                Yii::app()->cache->set($this->_modTimeCachePrefix . $pathCacheKey, null);
                                @unlink( $contentFile );
                                return false;
                        }
                }

        } else {
                return true;
        }
        // unset cache key
        Yii::app()->cache->set($this->_modTimeCachePrefix . $pathCacheKey, null);
        @unlink( $contentFile );
        return false;
    }

    /**
     * Take existed file from filesystem
     *
     * @param string $path content path
     *
     * @return bool|string
     */
    protected function loadObjectContentFromCache($path)
    {
        $contentFile = $this->contentFile($path);
        if ($this->fileAccessible($contentFile, $this->cacheFolder)) {
            $content = new Content($this, $path);
            return $content;
        } else {
            return false;
        }
    }

    /**
     * Create file,and save its last modification date
     *
     * @param string $path    string relative path
     * @param string $content string file content
     * @param int    $time    mixed last modification time
     *
     * @throws Exception
     * @return string string absolute path
     *
     */
    private function _createFile($path, $content, $time)
    {
        // create change key, add cachefolder in case you have different domains and equal paths
        $pathCacheKey = $this->cacheFolder.$path;

        $contentFile = $this->contentFile($path);
        if (file_exists($contentFile) && !$this->fileAccessible($contentFile, $this->cacheFolder)) {
            return false;
        }

        $dirPath = pathinfo($contentFile, PATHINFO_DIRNAME);
        if (!file_exists($dirPath)) {
            if (!@mkdir($dirPath, 0777, true)) {
                throw new Exception("Can't create dir $dirPath");
            }
        }
        if (file_put_contents($contentFile, $content) === false) {
            throw new Exception("Can't create file $contentFile");
        }
        if (!touch($contentFile, $time)) {
            throw new Exception("Can't touch() file $contentFile");
        }
        if (!chmod($contentFile, 0666)) {
            throw new Exception("Can't chmod() file $contentFile");
        }

        if (!$this->fileAccessible($contentFile, $this->cacheFolder)) {
            unlink($contentFile);
            return false;
        }

        // set updatetimestamp in cache for this object
        Yii::app()->cache->set($this->_modTimeCachePrefix . $pathCacheKey, $time); //, Yii::app()->getModule('contentSource')->cacheTime);

        return $contentFile;
    }

    /**
     * Get content file location
     *
     * @param string $path content path
     *
     * @return string
     */
    public function contentFile($path)
    {
        return $this->cacheFolder . $path;
    }


}
