(function($){
	
	/**
	 * Elements storage<br />
	 * When declaring and creating elements, they will be stored here
	 * @class Elements
	 */
	$.fg.Elements = {
			
			/**
			 * Declared elements (element type => element object)
			 * @type {object}
			 */
			elements: {},
			/**
			 * Storage of created elements in editor
			 * @type {array}
			 */
			storage: [],
		
		/**
		 * Declare element
		 * @param {string} elementType type of the element
		 * @param {object} elementObject element object(class)
		 * @return {void}
		 */
		add: function(elementType, elementObject)
		{
			this.elements[elementType] = elementObject;
		},
		
		/**
		 * Create element and add to storage if needed
		 * @param {string} elementType type of the element to create
		 * @param {boolean} addToStorage add element to elements storage or not
		 * @return {object} created element object
		 */
		create: function(elementType, addToStorage)
		{
			var elementObject = new (this.get(elementType))();
			var id = typeof addToStorage === "undefined" || addToStorage
				? this.storage.push( elementObject ) - 1
				: "";
			elementObject.setProperty( "id", id );
			elementObject.setProperty( "type", elementType );
			if(elementObject.hasProperty( "name" ) && elementObject.getProperty( "name" ).toString() === "") {
				elementObject.setProperty( "name", elementType+id );
			}
			
			return elementObject;
		},
		
		/**
		 * Get element from declared storage
		 * @param {string} elementType type of the element
		 * @return {object} element object(class)
		 */
		get: function(elementType)
		{
			return this.elements[elementType];
		},
		
		/**
		 * Get element from storage of created elements
		 * @param {number} elementId the id of the element
		 * @return {object} element object
		 */
		getFromStorage: function(elementId)
		{
			return this.storage[elementId] || null;
		}
		
	};
	
	/**
	 * Root class for all elements<br />
	 * Must be extended by all elements
	 * @class Element
	 */
	$.fg.Element = $.Class.extend({},
	/** @lends $.fg.Element */
	{
	
			/**
			 * Contains element properties that can be saved or edit by element editor
			 * @type {object}
			 */
			properties: null,
			/**
			 * Unique element id
			 * @type {number}
			 */
			id: 0,
			/**
			 * @type null / object
			 * @desc parent element that owns this element
			 */
			containerElement: null,
		
		/**
		 * Initialize the element
		 * @return {void}
		 */
		init: function()
		{
			//this.loadFromJSON( json );
		},
		
		/**
		 * Set element property value
		 * @param {string} name the name of the element property
		 * @param {string} value the value of the element property
		 * @return {void}
		 */
		setProperty: function(name, value)
		{
			if(this.properties && this.properties[name] && this.properties[name].handler) {
				switch(this.properties[name].handler) {
					case "boolean":
						value = value == "1"
						break;
				}
			}
			this[name] = value;
		},
		
		/**
		 * Get element property value
		 * @param {string} name the name of the element property
		 * @return {void}
		 */
		getProperty: function(name)
		{
			return typeof this[name] !== "undefined"
				? this[name]
				: null;
		},
		
		/**
		 * Is element has property
		 * @param {string} name the name of the element property
		 * @return {boolean}
		 */
		hasProperty: function(name)
		{
			return typeof this[name] !== "undefined";
		},
		
		/**
		 * Extend element properties
		 * @param {object} properties
		 * @return {void}
		 */
		extendProperties: function(props)
		{
			this.properties = $.extend(
				{},
				(this.properties || {}),
				props
			);
		},
		
		/**
		 * Render element property for element editor area
		 * @param {string} name the name of the element property
		 * @return {object} rendered property DOM node
		 */
		renderProperty: function(name)
		{
			var prop = this.properties[name];
			var elementProp = this.getProperty( name );
			var o;
			if(typeof prop.handler == "function") {
				o = prop.handler.call();
			} else {
				switch(prop.handler) {
					default:
					case "input":
						o = $( "<input />", {
							type: "text",
							value: elementProp,
							name: name
						} );
						break;
					case "boolean":
						o = $( "<select>", {
							name: name
						} )
							.append(
								$( "<option>" )
									.attr( "value", 0 )
									.attr( "selected", !elementProp )
									.text( "false" )
							)
							.append(
								$( "<option>" )
									.attr( "value", 1 )
									.attr( "selected", !!elementProp )
									.text( "true" )
							);
						break;
					case "dropdown":
						if(prop.listelements) {
							o = $( "<select>", {
								name: name
							} );
							$.each(prop.listelements, function(k, v){
								var opt = $( "<option>" )
									.attr( "value", v.value )
									.text( v.label );
								if(elementProp === v.value)
									opt.attr( "selected", true );
								o.append(opt);
							});
						}
						break;
					case "listbuilder":
						var listBuilder = new $.fg.ElementsHelper.listBuilder( elementProp );
						o = document.createDocumentFragment();
						o.append( listBuilder.render() );
						break;
				}
			}
			return o;
		},
		
		/**
		 * Set element DOM object attributes depending on element properties
		 * @param {object} element DOM object
		 * @return {void}
		 * @see $.fg.ElementEditor
		 */
		setElementProperties: function(element)
		{
			if(!this.properties)
				return;
			var that = this;
			$.each(this.properties, function(k, v){
				if(v.recipient) {
					switch(v.recipient) {
						case "attribute":
							element.attr( k, that.getProperty( k ) );
							break;
						case "css":
							element.css( k, that.getProperty( k ) );
							break;
						case "html":
							element.html( that.getProperty( k ) );
							break;
						case "text":
							element.text( that.getProperty( k ) );
							break;
					}
				}
			});
		},
		
		/**
		 * Get element assets url path
		 * @return {string} assets path
		 */
		getAssetsPath: function()
		{
			return "assets/elements/"+this.getProperty( "type" );
		},
		
		/**
		 * Load element asset file
		 * @param {string} path path to the asset
		 * @param {function} onloadCallback called when asset will be loaded
		 * @return {void}
		 */
		loadAsset: function(path, onloadCallback)
		{
			$.loadAsset( this.getAssetsPath() + "/" + path, onloadCallback );
			/*if(this.containerElement) {
				var src = $.fg.Config.basePath + "/" + this.getAssetsPath() + "/" + path;
				var script = $( "<script>", {type: "text/javascript", src: src} );
				this.containerElement.getRow( this.id ).get(0).appendChild( script.get(0) );
			}*/
		},
		
		/**
		 * Add script string into element row as last tag (used for compatibility when rendering HTML for frontend)
		 * @param {string} scriptText JavaScript code
		 * @return {void}
		 */
		addScriptDeclaration: function(scriptText)
		{
			if(this.containerElement) {
				var script = $( "<script>", {type: "text/javascript", text: "$(function(){"+ scriptText +"});"} );
				this.containerElement.getRow( this.id ).find( ".content" ).get(0).appendChild( script.get(0) );
			}
		},
		
		/**
		 * Abstract method<br />
		 * If element will implement this method, then it must return GUI for element editor area for this element
		 * @param {$.fg.ElementEditor} editor {@link $.fg.ElementEditor}
		 * @return {object} GUI DOM node
		 */
		edit: function(editor)
		{
			return null;
		},
	   
	   /**
		 * Abstract method<br />
		 * Draw element that will be appended to the form design area
		 * @return {object} DOM node
		 */
		render: function()
		{
			return null;
		},
		
		/**
		 * Abstract method<br />
		 * Calls after element is drawn
		 * @param {object} row DOM node of the row in which element is drawn
		 * @return {void}
		 */
		afterRender: function(row)
		{
			return null;
		},
		
		/**
		 * Abstract method<br />
		 * Draw element that will be appended to the library area as element representation
		 * @return {object} DOM object or null
		 */
		renderLibraryView: function()
		{
			return null;
		},
		
		/**
		 * Abstract method<br />
		 * Calls before form is saved<br />
		 * This needs to do some operations before form is saved, to remove some nodes that must not be saved for frontend form
		 * @param {object} formObject DOM object of the form
		 * @return {void}
		 * @example formObject.find( ".row .toolbar" ).remove();
		 */
		beforeSaveFormAsHTML: function(formObject)
		{
			return null;
		},
		
		/**
		 * Load element from object that contains element properties, this object coming from JSON serialized form
		 * @param {object} element object properties list
		 * @return {void}
		 */
		loadFromObject: function(object)
		{
			if(this.properties) {
				$.each( this.properties, $.proxy( function(k, v){
					if(typeof object[k] !== "undefined") {
						this.setProperty( k, object[k] );
					}
				}, this ) );
			}
		},
		
		/**
		 * Serialize element to object with properties that will be later stored as JSON
		 * @return {object} object with properties 
		 */
		toObject: function() {
			var data = {};
			data.type = this.type;
			if(this.properties) {
				$.each( this.properties, $.proxy( function(k, v){
					data[k] = this.getProperty( k );
				}, this ) );
			}
			return data;
		}
		
		/*toJSON: function()
		{
			return $.toJSON( this.toObject(), false );
		}*/
		
	});
	
	/**
	 * Root class for elements that can contain other elements, for example form element
	 * @class ElementsContainerElement
	 */
	$.fg.ElementsContainerElement = $.fg.Element.extend(
	/** @lends $.fg.ElementsContainerElement */
	{
			/**
			 * DOM object of element container area
			 * @type {object}
			 */
			object: null,
			/**
			 * Storage of elements that was added to this container
			 * @type {array}
			 */
			elements: [],
		
		/**
		 * When element rendering it's GUI it must fire this method on DOM element that will contain other elements
		 * @param {object} containerObject container DOM node
		 * @return {void}
		 */
		initContainer: function(containerObject)
		{
			this.object = $( containerObject );
			
			$( this.object ).droppable({
				greedy: true,
				drop: $.proxy( function(event, ui) {
					var elementType = ui.helper.attr( "element-type" );
					if(elementType) {
						var element = $.fg.Elements.create( elementType );
						var elementId = element.id;
						this.addRow( element );
						undoRedo(
							function() {
								var element = $.fg.Elements.getFromStorage( elementId );
								element.containerElement.deleteRow( elementId );
							},
							function() {
								var element = $.fg.Elements.getFromStorage( elementId );
								element.containerElement.addRow( element );
							}
						);
					}
				}, this )
			});
			$( this.object ).sortable({
				element: ".row",
				handle: ".toolbar li.sort",
				axis: "y",
				placeholder: "row-sortable-placeholder",
				forcePlaceholderSize: true,
				start: $.proxy( this.onSortStart, this ),
				update: $.proxy( this.onSortUpdate, this )
			});
		},
		
		/**
		 * Add element to container
		 * @param {object} element element object
		 * @return {void}
		 */
		addRow: function(element, rowPosition)
		{
			//var element = $.fg.Elements.create(elementType);
			var elementType = element.type;
			var elementId = element.getProperty( "id" );
			this.elements.push( element );
			element.setProperty( "containerElement", this );
			var row = $( "<div></div>" )
				.addClass( "row" )
				.addClass( "element-" + elementType )
				.attr( "element-id", elementId )
				.attr( "id", "form-row-" + elementId )
				.append( $( "<div class=\"content\"></div>" ) )
				.append(
					$( "<div class=\"toolbar\"></div>" )
						.append(
							$( "<ul></ul>" )
								.append(
									$( "<li>" ).addClass( "edit" ).attr( "title", t( "row-action-edit" ) )
										.append( $( "<a>" ).attr( "href", "#").text( t( "row-action-edit" ) ) )
								)
								.append(
									$( "<li>" ).addClass( "delete" ).attr( "title", t( "row-action-delete" ) )
										.append( $( "<a>" ).attr( "href", "#").text( t( "row-action-delete" ) ) )
								)
								.append(
									$( "<li>" ).addClass( "sort" ).attr( "title", t( "row-action-sort" ) )
										.append( $( "<a>" ).attr( "href", "#").text( t( "row-action-sort" ) ) )
								)
						)
				);
			this.object.append( row );
			this.updateRow( elementId );
			row.find( ".content" ).bind( "click", this.contentClickHandler );
			row.find( ".toolbar .edit a" ).bind( "click", this.editHandler );
			row.find( ".toolbar .delete a" ).bind( "click", this.deleteHandler );
			row.find( ".toolbar .sort a" ).bind( "click", this.sortHandler );
			
			if(typeof rowPosition !== "undefined") {
				this.sortRow( elementId, rowPosition );
			}
		},
		
		/**
		 * Render element in row
		 * @param {number} elementId id of the element in elements storage
		 * @return {void}
		 */
		updateRow: function(elementId)
		{
			var element = $.fg.Elements.getFromStorage( elementId );
			var row = this.getRow( elementId );
			row.find( ".content" ).text("").append( element.render() );
			//element.appendValidationField && element.appendValidationField();
			element.afterRender( row );
		},
		
		/**
		 * Remove element from container
		 * @param {number} elementId id of the element in elements storage
		 * @return {void}
		 */
		deleteRow: function(elementId)
		{
			this.getRow( elementId ).fadeOut(200, function(){
				$(this).remove();
			});
			
			var tmp = this.elements;
			this.elements = [];
			$.each( tmp, $.proxy( function(k, v) {
				if(v.id != elementId) {
					this.elements.push( v );
				}
			}, this ) );
			
			if($.fg.ElementEditor.currentElementId == elementId) {
				$.fg.ElementEditor.hide();
			}
		},
		
		/**
		 * Get row of the element
		 * @param {number} elementId id of the element in elements storage
		 * @return {object} DOM object of the row
		 */
		getRow: function(elementId)
		{
			return this.object.find( "#form-row-" + elementId );
		},
		
		sortRow: function(elementId, position)
		{
			var a = this.getRow( elementId );
			var b = this.object.find( ".row:eq("+position+")" );
			var aPosition = this.getRowPosition( elementId );
			var maxPos = this.object.find( ".row" ).length-1;
			
			if(aPosition == position) {
				return false;
			}
			
			if(aPosition == 0 || position == maxPos) {
				a.insertAfter( b );
			} else {
				a.insertBefore( b );
			}
			this.onAfterSortUpdate( aPosition, position );
			return true;
		},
		
		getRowPosition: function(elementId)
		{
			return this.getRow( elementId ).index()-1;
		},
		
		/**
		 * Reset container (remove all elements)
		 * @return {void}
		 */
		clear: function()
		{
			this.object.text( "" );
			this.elements = [];
			$.fg.ElementEditor.hide();
		},
		
		/**
		 * Overriden method
		 * @see $.fg.Element.addScriptDeclaration
		 */
		addScriptDeclaration: function(scriptText)
		{
			var script = $( "<script>", {type: "text/javascript", text: "$(function(){"+ scriptText +"});"} );
			this.object.get(0).appendChild( script.get(0) );
		},
		
		/**
		 * Handler of starting to sort of element
		 * @param {event} e
		 * @param {object} ui
		 * @return {void}
		 */
		onSortStart: function(e, ui)
		{
			$.data( ui.item, "prev-index", ui.item.index() );
		},
		
		/**
		 * Handler of element sorting is updated
		 * @param {event} e
		 * @param {object} ui
		 * @return {void}
		 */
		onSortUpdate: function(e, ui)
		{
			var prevIndex = $.data( ui.item, "prev-index" ) - 1;
			var newIndex = ui.item.index() - 1;
			var elementId = ui.item.attr( "element-id" );
			undoRedo(
				$.proxy( function() {
					this.sortRow( elementId, prevIndex );
				}, this ),
				$.proxy( function() {
					this.sortRow( elementId, newIndex );
				}, this )
			);
			this.onAfterSortUpdate( prevIndex, newIndex );
		},
		
		onAfterSortUpdate: function(prevIndex, newIndex)
		{
			//alert([prevIndex, newIndex]);
			//var prevIndex = $.data( ui.item, "prev-index" ) - 1;
			//var newIndex = ui.item.index() - 1;
			var tmp = this.elements[prevIndex+1];
			this.elements[prevIndex+1] = this.elements[newIndex+1];
			this.elements[newIndex+1] = tmp;
		},
		
		/**
		 * Handler of click on sort button
		 * @param {event} e
		 * @return {void}
		 */
		sortHandler: function(e)
		{
			e.preventDefault();
		},
		
		/**
		 * Handler of click on edit button
		 * @param {event} e
		 * @return {void}
		 */
		editHandler: function(e)
		{
			e.preventDefault();
			var elementId = $(e.target).closest( ".row" ).attr( "element-id" );
			$.fg.ElementEditor.edit( elementId );
		},
		
		/**
		 * Handler of click on delete button
		 * @param {event} e
		 * @return {void}
		 */
		deleteHandler: function(e)
		{
			e.preventDefault();
			if(confirm( t( "row-action-delete-confirm" ) )) {
				var elementId = $(e.target).closest( ".row" ).attr( "element-id" );
				var element = $.fg.Elements.getFromStorage( elementId );
				var rowPosition = element.containerElement.getRowPosition( elementId );
				//var elementJSON = element.toJSON();
				element.containerElement.deleteRow( elementId );
				undoRedo(
					function() {
						var element = $.fg.Elements.getFromStorage( elementId );
						element.containerElement.addRow( element, rowPosition );
					},
					function() {
						$.fg.Elements.getFromStorage( elementId ).containerElement.deleteRow( elementId );
					}
				);
			}
		},
		
		/**
		 * Handler of click on element
		 * @param {event} e
		 * @return {void}
		 */
		contentClickHandler: function(e)
		{
			var elementId = $(e.target).closest( ".row" ).attr( "element-id" );
			$.fg.ElementEditor.edit( elementId );
		}
		
	});
	
	/**
	 * Root class for elements that implementing form inputs (textfields, checkboxes, etc.)
	 * @class FormElement
	 */
	$.fg.FormElement = $.fg.Element.extend(
	/** @lends $.fg.FormElement */
	{
			/**
			 * HTML attribute "name"
			 * @type {string}
			 */
			name: "",
			/**
			 * Label of the element
			 * @type {string}
			 */
			label: "Label",
			/**
			 * Is element mandatory or not
			 * @type {boolean}
			 */
			mandatory: false,
			/**
			 * "?" icon text
			 * @type {string}
			 */
			info: "",
			/**
			 * Element mouseover text
			 * @type {string}
			 */
			tooltip: "",
			/**
			 * Validation rules for element available to select in element editor area<br />
			 * {boolean} "false" if no validation types used,<br />
			 * {string} "all" if all validation types used,<br />
			 * {array} of strings (validation types) to use several types
			 * @type {mixed}
			 */
			validationTypes: false,
			/**
			 * Selected validation type of element
			 * @type {string}
			 */
			validation: '',
		
		/**
		 * Initialize the element
		 * @return {void}
		 */
		init: function()
		{
			this._super();
			
			this.extendProperties({
				"name": {
					handler: "input",
					recipient: "attribute",
					displayName: t("property.name")
				},
				"label": {
					handler: "input",
					displayName: t("property.label")
				},
				"mandatory": {
					handler: "boolean",
					displayName: t("property.mandatory")
				},
				"info": {
					handler: "input",
					displayName: t("property.info")
				},
				"tooltip": {
					handler: "input",
					displayName: t("property.tooltip")
				},
				"validation": {
					handler: $.proxy(this.renderValidationProperty, this),
					displayName: t("property.validation")
				}
			});
		},
		
		/**
		 * Render element label
		 * @return {object} label DOM node
		 */
		renderLabel: function()
		{
			var label = this.getProperty( "label" );
			var labelposition = this.getProperty( "labelposition" );
			var info = this.getProperty( "info" );
			var cls = [];
			if(info)
				label = "<a href=\"#\" class=\"info\" title=\""+ info +"\"></a>" + label;
			if(this.getProperty( "mandatory" )) {
				label += "<span class=\"mandatory\">*</span>";
			}
			if(!labelposition || labelposition == "left") {
				label += ": ";
			}
			cls.push( "labelposition-"+ (labelposition || "left") );
			if(this.getProperty( "mandatory" )) {
				cls.push( "mandatory" );
			}
			return $( "<label class=\""+ cls.join( " " ) +"\">"+ label +"</label>" );
		},
		
		

		/**
		 * Set tooltip(mouseover) text to element
		 * @param {string} jQuery selector
		 * @return {void}
		 */
		appendTooltip: function(selector)
		{
			var tooltip = this.getProperty( "tooltip" );
			if(tooltip.toString() !== "") {
				this.addScriptDeclaration(
					'$( "'+ selector +'" ).live( "mouseover", function() {'
						+'$(this).attr( "title", "'+ this.getProperty( "tooltip" ) +'" );'
						+'$(this).tipsy( {'
							+'trigger: "manual",'
							+'gravity: $.fn.tipsy.autoWE,'
							+'opacity: 0.5'
						+'} )'
							+'.tipsy( "show" );'
					+'} )'
						+'.live( "mouseout", function() {'
							+'$(this).tipsy( "hide" );'
						+'} );'
				);
			}
		},
		
		/**
		 * Render validation selector GUI for element editor area
		 * @return {object} selector DOM node
		 */
		renderValidationProperty: function()
		{
			if(this.validationTypes) {
				var validation;
				if(this.validationTypes === "all") {
					validation = $.fg.validations;
				} else {
					$.each(this.validationTypes, function(k,v) {
						validation[v] = $.fg.validations[v] || v;
					});
				}
				validation = $.extend( {"": "-"}, validation );
				
				o = $( "<select>", {
					name: "validation"
				} );
				$.each(validation, $.proxy( function(k, v){
					var opt = $( "<option>" )
						.attr( "value", k )
						.text( v );
					if(this.validation && this.validation === k)
						opt.attr( "selected", true );
					o.append(opt);
				}, this ));
				return o;
			}
			return null;
		}
		
		/*appendValidationField: function()
		{
			var value = this.getProperty( "validation" ) === ""
				? ""
				: $.md5( this.getProperty( "validation" ) + $.fg.Editor.getForm().getProperty( "salt" ) );
			var field = $( "<input />", {
				type: "hidden",
				name: "validation["+ this.getProperty( "name" ) +"]",
				value: value
			} );
			this.containerElement.getRow( this.id ).find( ".content" ).append( field );
		}*/
		
	});
	
	/**
	 * @class $.fg.Elements.Form
	 */
	$.fg.Elements.add("form", /**
	 * @class $.fg.Elements.Form
	 */$.fg.ElementsContainerElement.extend(
	/** @lends $.fg.Elements.elements */
	{
			id: "",
			name: "",
			email: "",
			database: "",
			method: "post",
			salt: "123",
		
		init: function()
		{
			this._super();
			
			this.name = t( "elements.form.defaultName" );
			
			this.extendProperties({
				"name": {
					handler: "input",
					displayName: t("property.name")
				},
				"email": {
					handler: "input",
					displayName: t("property.email")
				},
				"database": {
					handler: "boolean",
					displayName: t("property.database")
				},
                                "id": {
                                        handler: "input",
                                        displayName: t("property.id"),
					readOnly: true
                                }

			});
		},
		
		render: function()
		{
			var o = $( "<form>", {
				method: this.getProperty( "method" )
			} );
			
			this.setElementProperties( o );
			this.initContainer( o );
			o.click( this.clickHandler );
			
			return o;
		},
		
		afterRender: function()
		{
			this.addScriptDeclaration(
				'$( "#fg-form .row label a.info" )'
				+'.live( "click", function(e){'
				+'	e.preventDefault();'
				+'})'
				+'.tipsy({'
				+'	trigger: "click",'
				+'	gravity: "w",'
				+'	live: true'
				+'});'
			);
		},
		
		clickHandler: function(e)
		{
			$.fg.Library.hide();
		},
		
		beforeSaveFormAsHTML: function(formObject)
		{
			formObject.find( ".row .toolbar" ).remove();
		}
		
	}));
	
	$.fg.Elements.add("space", $.fg.Element.extend({
			
		render: function()
		{
			return null;
		}
		
	}));
	
	$.fg.Elements.add("checkbox", $.fg.FormElement.extend({
		
			label: "Check",
			labelposition: "left",
			preselected: false,
			value: "1",
		
		init: function()
		{
			this._super();
			
			delete(this.properties.tooltip);
			
			this.extendProperties({
				"labelposition": {
					handler: "dropdown",
					listelements: [
						{
							value: "left",
							label: "left"
						},
						{
							value: "right",
							label: "right"
						}
					],
					displayName: t("property.labelposition")
				},
				"preselected": {
					handler: "boolean",
					displayName: t("property.preselected")
				},
				"value": {
					handler: "input",
					recipient: "attribute",
					displayName: t("property.value")
				}
			});
		},
		
		render: function()
		{
			var dF = document.createDocumentFragment();
			var o = $( "<input />" );
			o.attr( "type", "checkbox" );
			this.setElementProperties( o );
			if(this.getProperty( "preselected" )) {
				o.attr( "checked", true );
			}
			dF.append( o );
			if(this.getProperty( "labelposition" ) == "right") {
				o.after( this.renderLabel() );
			} else {
				o.before( this.renderLabel() );
			}
			return dF;
		},
		
		renderLibraryView: function()
		{
			var dF = document.createDocumentFragment();
			dF.append(
				$( "<input>", {
					type: "checkbox",
					checked: true
				} )
			);
			dF.append(
				$( "<label>", {
					text: t( "elements.checkbox.title" )
				} )
			);
			return dF;
		}
		
	}));
	
	$.fg.Elements.add("input", $.fg.FormElement.extend({
		
			value: "",
			validationTypes: "all",
		
		init: function()
		{
			this._super();
			
			this.label = t( "elements.input.defaultLabel" );
			
			this.extendProperties({
				"value": {
					handler: "input",
					recipient: "attribute",
					displayName: t("property.value")
				}
			});
		},
		
		render: function()
		{
			var dF = document.createDocumentFragment();
			var o = $( "<input />", {type: "text"} );
			this.setElementProperties( o );
			dF.append( this.renderLabel() );
			dF.append( o );
			return dF;
		},
		
		afterRender: function(row)
		{
			var rowId = row.attr( "id" );
			this.appendTooltip( "#"+rowId+" .content input" );
			
		},
		
		renderLibraryView: function()
		{
			return $( "<input>", {
				type: "text",
				value: t( "elements.input.title" )
			} );
		}
		
	}));
	
	$.fg.Elements.add("textarea", $.fg.FormElement.extend({
		
			label: "Text",
			value: "",
		
		init: function()
		{
			this._super();
			
			this.extendProperties({
				"value": {
					handler: "input",
					recipient: "text",
					displayName: t("property.value")
				}
			});
		},
		
		render: function()
		{
			var dF = document.createDocumentFragment();
			var o = $( "<textarea />", {
				text: t( "elements.textarea.title" )
			} );
			o.css( "resize", "none" );
			this.setElementProperties( o );
			dF.append( this.renderLabel() );
			dF.append( o );
			return dF;
		},
		
		afterRender: function(row)
		{
			var rowId = row.attr( "id" );
			this.appendTooltip( "#"+rowId+" .content textarea" );
	
		},
		
		renderLibraryView: function()
		{
			return $( "<textarea>", {
				text: t( "elements.textarea.title" ),
				css: {resize: "none"}
			} );
		}
		
	}));
	
	$.fg.Elements.add("text", $.fg.FormElement.extend({
		
			label: "Text",
			value: "",
		
		init: function()
		{
			this._super();
			
			this.extendProperties({
				/*"value": {
					handler: "textarea",
					recipient: "attribute"
				}*/
			});
		},
		
		render: function()
		{
			var dF = document.createDocumentFragment();
			var o = $( "<textarea />" );
			o.css( "resize", "none" );
			this.setElementProperties( o );
			dF.append( this.renderLabel() );
			dF.append( o );
			return dF;
		},
		
		afterRender: function(row)
		{
			this.initEditor();
			var rowId = row.attr( "id" );
			this.appendTooltip( "#"+rowId+" .content .cleditorMain" );
			
		},
		
		initEditor: function()
		{
			/*this.loadAsset( "js/cleditor/jquery.cleditor.min.js" );
			this.loadAsset( "js/cleditor/jquery.cleditor.css");
			var rowId = this.containerElement.getRow( this.id ).attr( "id" );
			this.addScriptDeclaration('$( "#'+ rowId +' .content textarea" ).cleditor();');*/
			var assetsPath = this.getAssetsPath();
			var rowId = this.containerElement.getRow( this.id ).attr( "id" );
			/*this.addScriptDeclaration(
				'$.loadAsset( "'+ assetsPath +'/js/cleditor/jquery.cleditor.min.js", function() {'
				+'	$.loadAsset( "'+ assetsPath +'/js/cleditor/jquery.cleditor.css", function() {'
				+'		$( "#'+ rowId +' .content textarea" ).cleditor()'
				+'} )'
				+'} );'
			);*/
			this.addScriptDeclaration(
				'$.loadAsset( ['
				+'	"'+ assetsPath +'/js/cleditor/jquery.cleditor.min.js"'
				+'	,"'+ assetsPath +'/js/cleditor/jquery.cleditor.css"'
				+'], function() {'
				+'	$( "#'+ rowId +' .content textarea" ).cleditor()'
				+'	});'
			);
		},
		
		beforeSaveFormAsHTML: function(formObject)
		{
			formObject.find( ".row .content .cleditorMain" ).each(function(k, editor) {
				var textarea = $(editor).find( "textarea" );
				textarea.show();
				$(editor).replaceWith(textarea);
			});
		}
		
	}));
	
	$.fg.Elements.add("next/previous", $.fg.Element.extend({
		
			//target: "",
		
		init: function()
		{
			this._super();
			
			/*this.extendProperties({
				"target": {
					type: "input"
				}
			});*/
		},
		
		render: function()
		{
			var dF = document.createDocumentFragment();
			dF.append(
				$( "<button />" )
					.addClass( "previous" )
					.text( t( "elements.next/previous.previous-label" ) )
			);
			dF.append(
				$( "<button />" )
					.addClass( "next" )
					.text( t( "elements.next/previous.next-label" ) )
			);
			return dF;
		},
		
		renderLibraryView: function()
		{
			var dF = document.createDocumentFragment();
			dF.append(
				$( "<button />" )
					.addClass( "previous" )
					.text( t( "elements.next/previous.previous-label" ) )
			);
			dF.append(
				$( "<button />" )
					.addClass( "next" )
					.text( t( "elements.next/previous.next-label" ) )
			);
			return dF;
		}
		
	}));
	
	$.fg.Elements.add("reset", $.fg.Element.extend({
		
		init: function()
		{
			this._super();
		},
		
		render: function()
		{
			var o = $( "<input>", {
				type: "reset",
				value: t( "elements.reset.text" )
			} );
			return o;
		},
		
		renderLibraryView: function()
		{
			var o = $( "<input>", {
				type: "reset",
				value: t( "elements.reset.text" )
			} );
			return o;
		}
		
	}));
	
	$.fg.Elements.add("submit", $.fg.Element.extend({
			
			thankyoupage: "",
			
		init: function()
		{
			this._super();
			
			this.extendProperties({
				"thankyoupage": {
					handler: "input",
					displayName: t("property.thankyoupage")
				}
			});
		},
		
		render: function()
		{
			var o = $( "<input>", {
				type: "submit",
				value: t( "elements.submit.text" )
			} );
			return o;
		},
		
		renderLibraryView: function()
		{
			var o = $( "<input>", {
				type: "submit",
				value: t( "elements.submit.text" )
			} );
			return o;
		}
		
	}));
	
	$.fg.Elements.add("captcha", $.fg.FormElement.extend({
			
			mandatory: true,
			
		init: function()
		{
			this._super();
			
			delete( this.properties["mandatory"] );
			
			this.label = t( "elements.captcha.defaultLabel" );
			
			this.extendProperties({
			});
		},
		
		getCaptchaImageUrl: function()
		{
			//return $.fg.Config.basePath + "/" + this.getAssetsPath() + "/captcha.php";
			//return $.fg.Editor.serverParams["assetsPath"] + "/elements/captcha/captcha.php";
			return $.fg.Config.serverFormActionsUrl.replace("{action}","captcha"); 
		},
		
		render: function()
		{
			var dF = document.createDocumentFragment();
			var inner = $( "<div>" ).addClass( "inner" );
			
			var img = $( "<img />" );
			
			var input = $("<input>", {
				
			});
			this.setElementProperties( input );
			
			var refresh = $( "<a>", {
				href: "#"
			} )
				.addClass( "refresh" );
			//refresh.click( $.proxy( this.refreshHandler, this ) );
			
			inner.append( img );
			inner.append( refresh );
			inner.append( input );
			dF.append( this.renderLabel() );
			dF.append( inner );
			return dF;
		},
		
		afterRender: function(row)
		{
			this.refreshImage();
			var rowId = row.attr( "id" );
			this.addScriptDeclaration(
				'$( "#'+ rowId +' .content .refresh" ).click( function(e) {'
					+'e.preventDefault();'
					+'$( "#'+ rowId +' .content img" ).attr( "src", "'+ this.getCaptchaImageUrl() +'?" + Math.random() )'
				+'} );'
			);
			this.appendTooltip( "#"+rowId+" .content input" );
			
		},
		
		renderLibraryView: function()
		{
			return $( "<img />", {
				src: this.getCaptchaImageUrl()
			} );
		},
		
		refreshImage: function(e)
		{
			var img = this.containerElement.getRow( this.id ).find( ".content img" );
			img.attr( "src", this.getCaptchaImageUrl() + "?" + Math.random() );
		},
		
		refreshHandler: function(e)
		{
			e.preventDefault();
			this.refreshImage();
		}
		
	}));
	
	/*$.fg.Elements.add("mandatory_group", $.fg.ElementsContainerElement.extend({
		
			elements: [],
		
		init: function()
		{
			this._super();
			
			this.extendProperties({
				"errorMessage": {
					type: "input"
				}
			});
		},
		
		render: function()
		{
			var o = $( "<fieldset>" );
			this.setElementProperties( o );
			o.append(
				$( "<legend>" )
					.text("Mandatory group")
			);
			
			this.initContainer( o );
			
			return o;
		}
		
	}));*/
	
	$.fg.Elements.add("dropdown", $.fg.FormElement.extend({
			
			listelements: [],
			
		init: function()
		{
			this._super();
			
			this.listelements = [
				{
					value: t( "elements.dropdown.title" ),
					label: t( "elements.dropdown.title" )
				}
			];
			
			this.extendProperties( {
				"listelements": {
					handler: "listbuilder",
					displayName: t("property.listelements")
				}
			} );
		},
		
		render: function()
		{
			var dF = document.createDocumentFragment();
			var o = $( "<select>" );
			if(this.listelements.length) {
				$(this.listelements).each( function(k, v) {
					o.append( $( "<option>", {value: v.value, text: v.label} ) )
				} );
			}
			this.setElementProperties( o );
			dF.append( this.renderLabel() );
			dF.append( o );
			return dF;
		},
		
		afterRender: function(row)
		{
			var rowId = row.attr( "id" );
			this.appendTooltip( "#"+rowId+" .content select" );
			
		},
		
		renderLibraryView: function()
		{
			var o = $( "<select>" )
				.append(
					$( "<option>", {
						text: t( "elements.dropdown.title" )
					} )
				);
			return o;
		}
		
	}));
	
	$.fg.Elements.add("radio", $.fg.FormElement.extend({
			
			listelements: [],
			
		init: function()
		{
			this._super();
			
			this.listelements = [
				{
					value: t( "elements.radio.title" ),
					label: t( "elements.radio.title" )
				}
			];
			
			delete(this.properties.tooltip);
			
			this.extendProperties( {
				"listelements": {
					handler: "listbuilder",
					displayName: t("property.listelements")
				}
			} );
		},
		
		render: function()
		{
			var dF = document.createDocumentFragment();
			dF.append( this.renderLabel() );
			if(this.listelements.length) {
				$(this.listelements).each( $.proxy( function(k, v) {
					dF.append(
						$( "<label>" )
							.text( v.label )
							.prepend( $( "<input>", {
								type: "radio",
								name: this.getProperty( "name" ),
								value: v.value
							} ) )
					);
				}, this ) );
			}
			return dF;
		},
		
		renderLibraryView: function()
		{
			var dF = document.createDocumentFragment();
			dF.append( $( "<input>", {
				type: "radio",
				checked: true
			} ) );
			dF.append( $( "<label>" , {
				text: t( "elements.radio.title" )
			} ) );
			return dF;
		}
		
	}));
	
	$.fg.Elements.add("matrix", $.fg.Element.extend({
			
			name: "",
			label: "New matrix",
			
		init: function()
		{
			this._super();
			
			this.extendProperties({
				"name": {
					handler: "input",
					displayName: t("property.name")
				},
				"label": {
					handler: "input",
					displayName: t( "elements.matrix.property-title-display-name" )
				}
			});
			
			this.questions = [];
			this.answers = [];
			
			this.questions.push( "Test q #1" );
			this.questions.push( "Test q #2" );
			
			this.answers.push( "answer #1" );
			this.answers.push( "answer #2" );
			this.answers.push( "answer #3" );
			this.answers.push( "answer #4" );
			this.answers.push( "answer #5" );
		},
		
		render: function()
		{
			var o = $( "<fieldset>" );
			//this.setElementProperties( o );
			o.append(
				$( "<legend>" )
					.text( this.getProperty( "label" ) )
			);
			
			var table = $( "<table>" );
			var header = $( "<tr>" );
			header.append( $( "<td>" ).html( "&nbsp;" ) );
			$(this.answers).each(function(k, v) {
				header.append( $( "<th class=\"answer\">" ).text( v ) );
			});
			table.append( header );
			
			$(this.questions).each($.proxy(
				function(k, v) {
					var tr = $( "<tr>" );
					tr.append( $( "<th class=\"question\">" ).text( v ) );
					$(this.answers).each($.proxy(function(kk, vv) {
						tr.append(
							$( "<td class=\"input\">" )
								.append(
									$( "<input>" )
										.attr( "type", "radio" )
										.attr( "name", this.getProperty( "name" )+"["+ k +"]" )
								)
						);
					}, this));
					table.append( tr );
				},
				this
			));
			
			o.append(table);
			
			return o;
		},
		
		edit: function(editor)
		{
			var hr = $( "<hr />" );
			var content = $( "<div>" ).addClass( "builder" );
			editor.getContent().append( hr, content );
			this._editArrayBuilder( "questions" );
			this._editArrayBuilder( "answers" );
		},
		
		_editArrayBuilder: function(propertyName)
		{
			var property = this[propertyName];
			var builder = $( "<div>" ).addClass( "box "+propertyName ).attr( "property-name", propertyName );
			var title = $( "<h4>" ).text( t( "elements.matrix."+propertyName ) );
			var buttonAdd = $( "<a>" ).addClass( "button-add" ).attr( "href", "#" ).click( this.addHandler );
			var ul = $( "<ul>" );
			title.append( buttonAdd );
			builder.append( title );
			builder.append( ul );
			$.fg.ElementEditor.object.find( ".content .builder" ).append( builder );
			
			if(property && typeof property === "object" && property.length) {
				$(property).each($.proxy(function(k, v) {
					this.editBuilderAddRow( propertyName, v );
				}, this));
			}
			
			ul.sortable({
				recipient: "a.button-move",
				axis: "y",
				update: this.onSortUpdate
			});
		},
		
		editBuilderAddRow: function(propertyName, inputValue)
		{
			var ul = $.fg.ElementEditor.object.find( ".content .builder .box."+ propertyName + " ul" );
			var li = $( "<li>" );
			var input = $( "<input>" )
				.attr( "type", "text" )
				.val( inputValue )
				.addClass( "input" )
				.change( this.onInputChange );
			var buttonMove = $( "<a>" ).addClass( "button-move" ).attr( "href", "#" ).click( this.sortHandler );
			var buttonDelete = $( "<a>" ).addClass( "button-delete" ).attr( "href", "#" ).click( this.deleteHandler );
			li.append( input );
			li.append( buttonMove );
			li.append( buttonDelete );
			ul.append( li );
		},
		
		editBuilderDeleteRow: function(propertyName, index)
		{
			var ul = $.fg.ElementEditor.object.find( ".content .box."+ propertyName +" ul" );
			if(ul.length) {
				ul.find( "li" ).eq(index).remove();
				this.updateArray( propertyName );
			}
		},
		
		updateArray: function(propertyName)
		{
			var inputs = $.fg.ElementEditor.object.find( ".content .box."+ propertyName +" ul input" )
			this[propertyName] = [];
			$.each( inputs, $.proxy(function(k, v) {
				this[propertyName].push( $(v).val() );
			}, this) );
			$.fg.ElementEditor.updateElement();
		},
		
		onSortUpdate: function(event, ui)
		{
			var propertyName = ui.item.closest( ".box" ).attr( "property-name" );
			$.fg.ElementEditor.getCurrentElement().updateArray( propertyName );
		},
		
		onInputChange: function(event)
		{
			var t = $(event.target);
			var propertyName = t.closest( ".box" ).attr( "property-name" );
			$.fg.ElementEditor.getCurrentElement().updateArray( propertyName );
		},
		
		sortHandler: function(event)
		{
			event.preventDefault();
		},
		
		deleteHandler: function(event)
		{
			event.preventDefault();
			var t = $(event.target);
			$.fg.ElementEditor.getCurrentElement().editBuilderDeleteRow( t.closest( ".box" ).attr( "property-name" ), t.closest( "li" ).index() );
		},
		
		addHandler: function(event)
		{
			event.preventDefault();
			$.fg.ElementEditor.getCurrentElement().editBuilderAddRow( $(event.target).closest( ".box" ).attr( "property-name" ) );
		},
		
		loadFromObject: function(object)
		{
			this._super( object );
			this.questions = object.questions;
			this.answers = object.answers;
		},
		
		toObject: function() {
			var data = this._super(  );
			data.questions = this.questions;
			data.answers = this.answers;
			return data;
		}
		
	}));
	
})(jQuery);
